package com.zcxk.rmcp.core.dao;

import com.zcxk.rmcp.api.dto.alarm.AlarmRuleDto;
import com.zcxk.rmcp.api.dto.alarm.AlarmRulePageDto;
import com.zcxk.rmcp.api.dto.alarm.ConfigDataDto;
import com.zcxk.rmcp.api.dto.alarm.RuleMeasuringDto;
import com.zcxk.rmcp.core.entity.DeviceAlarmRule;
import com.zcxk.rmcp.core.entity.TplMeasuringDataDef;
import org.apache.ibatis.annotations.Param;


import java.util.List;

import org.apache.ibatis.annotations.Mapper;

/**
 * 设备告警规则(DeviceAlarmRule)表数据库访问层
 *
 * @author hym
 * @since 2021-07-30 10:17:05
 */
@Mapper
public interface DeviceAlarmRuleMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    DeviceAlarmRule selectById(Integer id);


    /**
     * 查询全部
     *
     * @return 对象列表
     */
    List<DeviceAlarmRule> selectAll();

    /**
     * 通过实体作为筛选条件查询
     *
     * @param deviceAlarmRule 实例对象
     * @return 对象列表
     */
    List<AlarmRuleDto> selectList(AlarmRulePageDto deviceAlarmRule);

    /**
     * 新增数据
     *
     * @param deviceAlarmRule 实例对象
     * @return 影响行数
     */
    int insert(DeviceAlarmRule deviceAlarmRule);

    /**
     * 批量新增
     *
     * @param deviceAlarmRules 实例对象的集合
     * @return 影响行数
     */
    int batchInsert(@Param("deviceAlarmRules") List<DeviceAlarmRule> deviceAlarmRules);

    /**
     * 修改数据
     *
     * @param deviceAlarmRule 实例对象
     * @return 影响行数
     */
    int update(DeviceAlarmRule deviceAlarmRule);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(Integer id);

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    int count();


    void delete(List<Integer> ids);

    void deleteAll(Integer alarmTypeId);

    List<RuleMeasuringDto> getRuleMeasuringList( Integer deviceTypeId);

    List<ConfigDataDto> getMeasuringOptionList(@Param("deviceTypeId")Integer deviceTypeId,
                                               @Param("measuringId") Integer measuringId);

    int findByNameUnique(@Param("id")Integer id,@Param("ruleName") String ruleName,
                         @Param("alarmTypeId") Integer alarmTypeId);

    int countMeasuringCode(@Param("id")Integer id,@Param("alarmTypeId") Integer alarmTypeId,
                           @Param("measuringCode") String measuringCode);

    DeviceAlarmRule findById(Integer id);

    TplMeasuringDataDef findTplMeasuringDataDef(@Param("deviceType")Integer deviceType, @Param("measuringCode")String measuringCode, @Param("value")String value);

}
