package com.zcxk.rmcp.core.dao;

import com.zcxk.core.common.pojo.UserCondition;
import com.zcxk.rmcp.api.dto.measurementsettlement.MeasurementSettlementDto;
import com.zcxk.rmcp.core.entity.MeasurementInstance;
import org.apache.ibatis.annotations.Param;


import java.util.List;

import org.apache.ibatis.annotations.Mapper;

/**
 * (MeasurementInstance)表数据库访问层
 *
 * @author hym
 * @since 2021-07-28 16:27:27
 */
@Mapper
public interface MeasurementInstanceMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    MeasurementInstance selectById(Integer id);


    /**
     * 查询全部
     *
     * @return 对象列表
     */
    List<MeasurementInstance> selectAll();

    /**
     * 通过实体作为筛选条件查询
     *
     * @param measurementSettlementDto 实例对象
     * @param userCondition
     * @return 对象列表
     */
    List<MeasurementSettlementDto> selectList(@Param("measurement")MeasurementSettlementDto measurementSettlementDto,
                                              @Param("userCondition")UserCondition userCondition);

    /**
     * 新增数据
     *
     * @param measurementInstance 实例对象
     * @return 影响行数
     */
    int insert(MeasurementInstance measurementInstance);

    /**
     * 批量新增
     *
     * @param measurementInstances 实例对象的集合
     * @return 影响行数
     */
    int batchInsert(@Param("measurementInstances") List<MeasurementInstance> measurementInstances);

    /**
     * 修改数据
     *
     * @param measurementInstance 实例对象
     * @return 影响行数
     */
    int update(MeasurementInstance measurementInstance);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(Integer id);

    /**
     * 查询总数据数
     *
     * @return 数据总数
     */
    int count();

    /**查询某天结算记录详情
      * @param id
 * @param lastClearingDay
     * @description
     * @author hym
     * @updateTime 2021/8/3 9:47
     * @return com.zcxk.rmcp.core.entity.MeasurementInstance
     * @throws
     */
    MeasurementInstance findClearingRecord(@Param("id") Integer id,
                                           @Param("lastClearingDay")  Integer lastClearingDay);


    /**修改已经审核的水表数目
      * @param size
 * @param id
     * @description
     * @author hym
     * @updateTime 2021/8/3 9:47
     * @return void
     * @throws
     */
    void modifyTheNumberOfAudits(@Param("size") int size,@Param("id") Integer id);
    /**修改所有已经审核的水表数目
      * @param id
     * @description
     * @author hym
     * @updateTime 2021/8/3 9:47
     * @return void
     * @throws
     */
    void modifyAllTheNumberOfAudits(Integer id);
}
